//ATmega324PB
//часы ds3231  wire
//прием температуры-влажности с датчика "GHT"
//приемник HC-12 на порту UART0
//отсылка данных для контроля через UART2
//разъем P2
/*
 1 GND
 2 Vcc
 3 Vo - регулятор подсветки
 4 RS ->  PD5
 5 R/W -> PD6
 6 E ->   PD7
 7 D0 -> PB0
 8 D1 -> PB1
 9 D2 -> PB2
10 D3 -> PB3
11 D4 -> PB4
12 D5 -> PB5
13 D6 -> PB6
14 D7 -> PB7
15 Vcc - подсветка
16 GND - подсветка
*/

#define F_CPU  14745600UL

//#define LED0 (1<<5)

#define BUAD  9600
#define BRC   ((F_CPU/16/BUAD) - 1) 

#include <Wire.h>
#include "lcd_proc.c" //текст процедур LCD на вкладке

volatile byte second, minute, hour, date, month, year;
volatile byte sek=0; //внутренний отсчет секунд
volatile byte Flag=0;
volatile byte count_rx=0; //счетчик принятых байт (должно быть 7)
volatile byte arr[7]; //приемный массив 
volatile int value=0; //целая температура в десятых градуса 
volatile byte hum=0; //влажность

void setup() {
  Wire.begin();  // Join i2c bus
//  DDRD  |= LED0;
//  PORTD |= LED0;
  DDRD |= (1<<5)|(1<<6)|(1<<7); //PD5,PD6,PD7 - на выход
  DDRB |= B11110000; //PB4-PB7 - на выход
  lcd_init();
  clearlcd(); //очистка экрана
  setpos(0,0); //нулевая позиция, верхняя строка 
  for (byte i=0; i<10; i++) 
  {byte ch=i+0x30; char_lcd(ch);}
  setpos(0,1); //нулевая позиция, нижняя строка
  str_lcd("ATmega328pb");
  delay (2000);
  clearlcd();
  setpos(0,0);
  str_lcd("Wait..");
  
  
//заводим порт UART2 для контрольной отсылки в компьютер
  UBRR2H = (unsigned char) (BRC >> 8);  // порт UART2, скорость = BUAD
  UBRR2L = (unsigned char)  BRC; 
  UCSR2B = (1 << TXEN); //разрешение передатчика
  UCSR2C = (1 << UCSZ1) | (1 << UCSZ0); //8 бит 1 стоп

//заводим порт UART0 для для приема данных через HC-12
  UBRR0H = (unsigned char) (BRC >> 8);  // порт UART2, скорость = BUAD
  UBRR0L = (unsigned char)  BRC; 
  UCSR0B = (1 << TXEN)|(1 << RXEN)|(1 << RXCIE); //передатчик-приемник+прерывание прием окончен
  UCSR0C = (1 << USBS) | (1 << UCSZ1) | (1 << UCSZ0);
//обнуляем приемный массив 
    for (byte i = 0; i<7; i++) arr[i]=0;

   EICRA |= (1<<ISC21);//включим прерывания INT2 по нисходящему фронту
   EIMSK |= (1<<INT2);   //разрешим внешние прерывания INT0 
}//end setup

ISR(USART0_RX_vect) //прерывание прием окончен
{
  arr[count_rx]=UDR0;
  if (arr[count_rx]==';') {
    Flag=3; count_rx=0; //конец приема будем обрабатывать
    if ((arr[0] == 'G') && (arr[1] == 'H') && (arr[2] == 'T')) //если первые три символа = "GHT"
    {
       value = arr[4] * 256 + arr[3]; // формируем двухбайтовое число
       value = value-2731; //целое число десятых градуса со знаком
       hum=arr[5];      
    }
  } else count_rx++;
}


 ISR(INT2_vect) //прерывание INT2 (вывод PB2)
{
//   PORTD ^= LED0; // мигание светодиодом LED1
   sek++; //внуренний отсчет секунд
   if (sek%2==0) //мигалка двоеточием
   {setpos(11,0); //11 позиция, верхняя строка 
    char_lcd(':');} //четная секунда - двоеточие
    else
   {setpos(11,0); //11 позиция, верхняя строка 
    char_lcd(' ');} //нечетная - пробел
    
   if (sek==60) sek=0;  
//   if (sek%4==0) Flag=2; //каждые 4 сек будем выводить
   if (sek==0) Flag=1; //в в начале минуты прочтем часы и синхронизируемся
   
 } 
void out_uart2(byte bb) //посылка UART2 с ожиданием готовности
{
  while (!bitRead(UCSR2A,UDRE));//ожидаем очистки бита UDRE
  UDR2=bb;
}

static uint8_t bcd2bin (uint8_t val) { return val - 6 * (val >> 4); }

void loop() {
 if (Flag==1) { //чтение часов каждую минуту
    Flag=0; //только один раз
    Wire.beginTransmission(0x68);                 // Start I2C protocol with DS1307 address
    Wire.write(0);                                // Send register address
    Wire.endTransmission(false);                  // I2C restart
    Wire.requestFrom(0x68, 7);                    // Request 7 bytes from DS1307 and release I2C bus at end of reading
    second = bcd2bin(Wire.read());                         // Read seconds from register 0
    minute = bcd2bin(Wire.read());                         // Read minuts from register 1
    hour   = bcd2bin(Wire.read());                         // Read hour from register 2
    Wire.read();                                  // Read day from register 3 (not used)
    date   = bcd2bin(Wire.read());                         // Read date from register 4
    month  = bcd2bin(Wire.read());                         // Read month from register 5
    year   = bcd2bin(Wire.read());                         // Read year from register 6
    sek=second; //синхронизация секунд
 } //end Flag=1

 if (Flag==3) { //при получении данных выводим на LCD
    Flag=0;
    byte t_10;
  clearlcd();
  setpos(0,0); //нулевая позиция, верхняя строка 
//выводим дату и время с ведущими нулями:
  t_10=date/10; //старший data
  char_lcd(t_10+0x30);
  t_10=date%10; //младший data
  char_lcd(t_10+0x30);
  char_lcd('.'); //точка
  t_10=month/10; //старший month
  char_lcd(t_10+0x30);
  t_10=month%10; //младший month
  char_lcd(t_10+0x30);
  char_lcd('.');  //точка
  t_10=year/10; //старший год
  char_lcd(t_10+0x30);
  t_10=year%10; //младший год
  char_lcd(t_10+0x30);

char_lcd(0x20); //пробел

  t_10=hour/10; //старший час
  char_lcd(t_10+0x30);
  t_10=hour%10; //младший час
  char_lcd(t_10+0x30);
  char_lcd(':');
  t_10=minute/10; //старший мин
  char_lcd(t_10+0x30);
  t_10=minute%10; //младший мин
  char_lcd(t_10+0x30);

 setpos(0,1); //нулевая позиция, нижняя строка 

//выводим температуру и влажность:
    if (value<0) char_lcd('-');
    else char_lcd('+');
    //3-разрядное число value превращаем в действительное с фиксированной точкой
    value=abs(value); //отбрасываем знак
    t_10=value/100; //старший
    if (t_10!=0) char_lcd(t_10+0x30);
    t_10 = value/10;  
    char_lcd(t_10%10+0x30); //средний
    char_lcd('.'); //отделяем десятые
    t_10=value%10;
    char_lcd(t_10+0x30); //младший
    char_lcd(0xDF); //градус для LCD (°)
    char_lcd('C'); 
    char_lcd(0x20); //пробел
    t_10=hum/10; //старший влажн
    char_lcd(t_10+0x30);
    t_10=hum%10; //младший влажн
    char_lcd(t_10+0x30);
    char_lcd('%');
  
 } //end Flag3

/*
if (Flag==2) { //каждые 4 сек выводим
  Flag=0; //только один раз
byte t_10;
//выводим с ведущими нулями:
t_10=date/10; //старший data
out_uart2(t_10+0x30);
  t_10=date%10; //младший data
out_uart2(t_10+0x30);
out_uart2('.'); //точка
t_10=month/10; //старший month
out_uart2(t_10+0x30);
  t_10=month%10; //младший month
out_uart2(t_10+0x30);
out_uart2('.');  //точка
t_10=year/10; //старший год
out_uart2(t_10+0x30);
  t_10=year%10; //младший год
out_uart2(t_10+0x30);

out_uart2(0x20); //пробел

t_10=hour/10; //старший час
out_uart2(t_10+0x30);
  t_10=hour%10; //младший час
out_uart2(t_10+0x30);
out_uart2(':');
t_10=minute/10; //старший мин
out_uart2(t_10+0x30);
  t_10=minute%10; //младший мин
out_uart2(t_10+0x30);
out_uart2(':');
t_10=sek/10; //старший sec
out_uart2(t_10+0x30);
  t_10=sek%10; //младший sec
out_uart2(t_10+0x30);

 out_uart2(0x0A); //перевод строки 0x0A
 out_uart2(0x0D); //возврат каретки 0x0D
 }//end Flag=2
 
 if (Flag==3) {
  byte t_10;
    //выводим температуру:
    if (value<0) out_uart2('-');
    else out_uart2('+');
    //3-разрядное число value превращаем в действительное с фиксированной точкой
    t_10=value/100; //старший
    if (t_10!=0) out_uart2(t_10+0x30);
    t_10 = value/10;  
    out_uart2(t_10%10+0x30); //средний
    out_uart2('.'); //отделяем десятые
    t_10=value%10;
    out_uart2(t_10+0x30); //младший
    out_uart2(0xB0); //стандартный ASCII-градус (°)
    out_uart2(0x20); //пробел
    t_10=hum/10; //старший влажн
    out_uart2(t_10+0x30);
    t_10=hum%10; //младший влажн
    out_uart2(t_10+0x30);
    out_uart2('%');

    out_uart2(0x0A); //перевод строки 0x0A
    out_uart2(0x0D); //возврат каретки 0x0D
    Flag=0;
 } //end Flag=3
 */
}//end loop
