
#define F_CPU 14745600UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

inline void set_e()    { PORTD |= (1 << 7); }
inline void clear_e() { PORTD &= ~(1 << 7); }
inline void set_rs()  { PORTD |= (1 << 5); }
inline void clear_rs()  { PORTD &= ~(1 << 5); }

void sendhalfbyte(unsigned char c)
{
  set_e(); // начальная установка
  _delay_us(50);
  PORTB = (PORTB & 0x0F) | (c << 4);
  clear_e(); 
  _delay_us(50);
}

void sendbyte(unsigned char c)
{
  sendhalfbyte(c >> 4);
  sendhalfbyte(c);
}

void setpos(unsigned char x, unsigned char y)
{
  clear_rs();
  sendbyte(((y << 6) + x) | 0x80);
}

void lcd_init(void)
{
  clear_rs();
  _delay_ms(40); // 15 - 80 мс
  sendhalfbyte (0x03);
  _delay_ms(5); 
  sendhalfbyte (0x03);
  _delay_us(100);
  sendhalfbyte (0x03);
  _delay_ms(2); 
  sendhalfbyte (0x02);
  _delay_ms(2);
  sendbyte(0x28); // 4-битный режим, 2 строки
  _delay_ms(2);
  sendbyte(0x0C); // включение дисплея, курсор запрещен
  _delay_ms(2);
  sendbyte(0x06); // курсор
  _delay_ms(2);
}

void clearlcd()
{
  clear_rs();
  sendbyte(0x01); // очистка экрана
  _delay_us(1500); 
}

void str_lcd (const char* str)
{
  set_rs();
  for(unsigned char n=0; str[n]; n++)
  sendbyte(str[n]);
}

void char_lcd (const uint8_t ch)
{
  set_rs();
  sendbyte(ch);
}


